import QtQuick 2.0
import QtQuick.XmlListModel 2.0

import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItems
import Ubuntu.Components.Popups 0.1

import "databasemodule.js" as DB

Page {

    ListView {
        id: feedList

        function reloadFeeds() {
            feedListModel.clear()
            DB.loadFeeds()
            var feeds = DB.dbResult
            for (var i = 0; i < feeds.rows.length; i++) {
                feedListModel.append({"link":feeds.rows.item(i).link,
                                         "displayName":feeds.rows.item(i).displayName,
                                         "siteLink":feeds.rows.item(i).siteLink,
                                         "description":feeds.rows.item(i).description})
            }
        }

        function infoLoadedByModel(url, title, siteLink, description) {
            DB.updateWithInfo(url, title, siteLink, description)
            reloadFeeds()
        }

        function loadFailedByModel(url) {
            console.log("LOAD FAILED FOR", url)
            DB.deleteFeed(url)
            feedList.reloadFeeds()
        }

        anchors.fill: parent
        model: ListModel {
            id: feedListModel
        }

        header: ListItems.Subtitled {
            text: i18n.tr("All news")
            subText: i18n.tr("View news from all channels")
            progression: true
            visible: feedListModel.count != 0

            onClicked: {
                console.log("Show all feeds")
            }
        } // header

        delegate: ListItems.Subtitled {
            text: (model.displayName === "")? model.link : model.displayName
            subText: model.description
            progression: true

            onPressAndHold: {
                feedList.currentIndex = model.index
                PopupUtils.open(popoverComponent, feedList)
            }

            onClicked: {
                console.log("TODO Show news in feed:", model.link)
            }
        } // delegate

        Component.onCompleted: {
            reloadFeeds()
            helperModel.infoLoaded.connect(infoLoadedByModel)
            helperModel.loadFailed.connect(loadFailedByModel)
        }

    } // ListView

    /* Thin wrapper around XMLHttpRequest.
     */
    QtObject {
        id: thinXmlHttpRequestWraper

        signal xmlLoaded(string xml)

        function loadXml(xmlUrl) {
            var doc = new XMLHttpRequest();
            doc.onreadystatechange = function() {
                if (doc.readyState == XMLHttpRequest.DONE) {
                    if (doc.status == 200 ) {
                        xmlLoaded(doc.responseText)
                    } else {
                        console.log("XMLHttpRequest ERROR", doc.status, doc.statusText)
                    }
                }
            }

            doc.open("GET", xmlUrl);
            doc.send();
        }

        onXmlLoaded: {
            helperModel.xml = xml
            // Another functions, such as saving in database and so on.
        }
    } // QtObject

    /* I really want remove this model. It is loading whole XML only for
     * getting feed information. And I haven't found any way to save loaded XML
     * to database, what we are want to do for offline reading (I suppose)...
     * UPDATE Now this helper model only parses data, downloaded by
     * thinXmlHttpRequest item. It works.
     */
    XmlListModel {
        id: helperModel

        property string currentUrl: ""

        signal infoLoaded(string url, string title, string siteLink, string description)
        signal loadFailed(string url)

        query: "/rss/channel"

        XmlRole { name: "title"; query: "title/string()" }
        XmlRole { name: "link"; query: "link/string()" }
        XmlRole { name: "description"; query: "description/string()" }

        onStatusChanged: {
            var firstItem
            switch(status)
            {
            case XmlListModel.Error:
                console.log("XmlListModel", "ERROR", errorString())
                loadFailed(source)
                break
            case XmlListModel.Ready:
                console.log("XmlListModel", "READY", "COUNT", count)
                if (count === 1) {
                    firstItem = helperModel.get(0)
                    infoLoaded(currentUrl, firstItem.title, firstItem.link, firstItem.description)
                    currentUrl = "" // todo mb no need
                } else { // if (count === 0 ) {
                    loadFailed(currentUrl)
                }

                break
            case XmlListModel.Loading:
                console.log("XmlListModel", "LOADING")
                break
            case XmlListModel.Null:
                console.log("XmlListModel", "NULL")
                break
            }
        }
    } // XmlListModel

    Component {
        id: popoverComponent

        Popover {
            id: popover
            Column {
                id: containerLayout
                anchors {
                    left: parent.left
                    top: parent.top
                    right: parent.right
                }

                ListItems.Standard {
                    text: i18n.tr("Update")
                    onClicked: {
                        PopupUtils.close(popover)
                        console.log("TODO Update")
                    }
                }

                ListItems.Standard {
                    text: i18n.tr("Edit")
                    onClicked: {
                        PopupUtils.close(popover)
                        console.log("TODO Edit")
                    }
                }

                ListItems.Standard {
                    text: i18n.tr("Open site")
                    onClicked: {
                        PopupUtils.close(popover)
                        Qt.openUrlExternally(feedListModel.get(feedList.currentIndex).siteLink)
                    }
                }

                ListItems.Standard {
                    text: i18n.tr("Remove")
                    onClicked: {
                        PopupUtils.close(popover)
                        DB.deleteFeed(feedListModel.get(feedList.currentIndex).link)
                        if (DB.dbResult.rowsAffected > 0)
                            feedListModel.remove(feedList.currentIndex)
                    }
                }

                ListItems.Standard {
                    text: i18n.tr("Remove all")
                    onClicked: {
                        PopupUtils.close(popover)
                        DB.dropTable()
                        feedListModel.clear()
                    }
                }
            } // Column
        } // Popover
    } // Popover component

    Label {
        id: lblEmptyBase
        text: i18n.tr("There are no RSS feeds to show")
        anchors.centerIn: parent
        visible: feedListModel.count == 0
    }

    ActivityIndicator {
        visible: helperModel.status == XmlListModel.Loading
        running: visible

        anchors.centerIn: parent
    }

    Row {

        anchors {
            bottom: parent.bottom
            horizontalCenter: parent.horizontalCenter
            margins: units.gu(2)
        }
        spacing: units.gu(2)


        TextField {
            id: tflinkEnter
            text: "http://news.yahoo.com/rss/"
        }

        // Temporary buttons: I just need toolbar icons...
        // And first time is much more faster than swipe to show toolbar.
        Button {
            id: btnAppendFeed

            text: i18n.tr("Append")
            width: units.gu(15)


            onClicked: {
                DB.addFeed(tflinkEnter.text)
                feedList.reloadFeeds() // Temporary and slow reloading.
                helperModel.currentUrl = tflinkEnter.text
                // helperModel.source = tflinkEnter.text
                thinXmlHttpRequestWraper.loadXml(tflinkEnter.text)
            }
        } // Button
    } // Row
} // Page
